# -*- coding: utf-8 -*-
from django.conf import settings


CROCDOC_API_KEY = getattr(settings, 'CROCDOC_API_KEY', None)
if CROCDOC_API_KEY is None:
    raise Exception("You must specify a CROCDOC_API_KEY in settings.py")


# -*- coding: utf-8 -*-
from django.views.generic import View
from braces.views import JSONResponseMixin

from .services import CrocodocWebhookService

import logging
logger = logging.getLogger('django.request')


class CrocdocCallbackView(JSONResponseMixin, View):
    """
    Handle the crocdoc callback
    """
    template = None
    json_dumps_kwargs = {'indent': 3}

    def get(self, request, *args, **kwargs):
        context_dict = {
            'message': 'Please Post to this endpoint',
        }
        return self.render_json_response(context_dict)

    def post(self, request, *args, **kwargs):
        service = CrocodocWebhookService(payload=request.POST.get('payload', '[]'))

        logger.info('recieved crocdoc webhook: {json}'.format(json=service.items))

        """
        status: payload
        [{"status": "DONE", "viewable": true, "event": "document.status", "uuid": "a2b9cdc4-50cc-466f-afd9-a37012dd1395"}]'

        comment: payload
        [{"uuid": "65814418-d47b-cee9-988f-370c248faa90", "doc": "b15532bb-c227-40f6-939c-a244d123c717", "page": 1, "owner": "2,Ross C", "type": "point", "event": "annotation.create"}, {"content": "test", "doc": "b15532bb-c227-40f6-939c-a244d123c717", "uuid": "32c6fd5d-551c-45f8-4c93-908b34923868", "owner": "2,Ross C", "event": "comment.create"}]

        ANNOTATION.CREATE

        textbox: payload
        [{"uuid": "86941091-97cb-43b0-4e2a-28a5457eb8da", "doc": "b15532bb-c227-40f6-939c-a244d123c717", "page": 1, "content": "fdafasfsddsa", "owner": "2,Ross C", "type": "textbox", "event": "annotation.create"}]

        highlight: payload
        [{"uuid": "114e0875-2f3f-2cb4-2516-fe464895ceaf", "doc": "b15532bb-c227-40f6-939c-a244d123c717", "page": 1, "content": "salary is \\u00a360k per annum. Salary", "owner": "2,Ross C", "type": "highlight", "event": "annotation.create"}]'

        ANNOTATION.UPDATE

        textbox: payload
        [{"uuid": "86941091-97cb-43b0-4e2a-28a5457eb8da", "doc": "b15532bb-c227-40f6-939c-a244d123c717", "page": 1, "content": "fdafasfsddsa", "owner": "2,Ross C", "type": "textbox", "event": "annotation.create"}]

        highlight: payload
        [{"uuid": "114e0875-2f3f-2cb4-2516-fe464895ceaf", "doc": "b15532bb-c227-40f6-939c-a244d123c717", "page": 1, "content": "salary is \\u00a360k per annum. Salary", "owner": "2,Ross C", "type": "highlight", "event": "annotation.create"}]'

        """
        context_dict = {
            'details': service.process(),
        }

        return self.render_json_response(context_dict)


# -*- coding: utf-8 -*-
from django.db import models

from .mixins import CrocodocService

from uuidfield import UUIDField
from jsonfield import JSONField


class CrocodocDocument(models.Model):
    uuid = UUIDField(hyphenate=True, blank=True, null=True, db_index=True)
    # content type of the object, used to do lookups as .source_object
    content_object_type = models.ForeignKey('contenttypes.ContentType', db_index=True)
    # id of the specific object
    object_id = models.IntegerField(db_index=True)
    # used to specify the name of the FileField
    object_attachment_fieldname = models.CharField(max_length=255, default='attachment')
    # non required connected reviewer user
    reviewer = models.ForeignKey('auth.User', null=True, blank=True)
    # misc data
    data = JSONField(default={}, blank=True, null=True)

    _crocodoc_service = None

    def get_url(self):
        """
        Return the local name of the object as it will probably be downloaded from there
        """
        return getattr(self.source_object, self.object_attachment_fieldname, 'attachment').name

    @property
    def crocodoc_service(self):
        if self._crocodoc_service is None:
            #
            # Pass in the source object with the attachment field
            #
            self._crocodoc_service = CrocodocService(attachment=self,
                                                     source_object=self.source_object,
                                                     attachment_field_name=self.object_attachment_fieldname)
        return self._crocodoc_service

    @property
    def source_object(self):
        """
        Return the object refered to by content_object_type and content_object_id
        """
        return self.content_object_type.get_object_for_this_type(pk=self.object_id)



"""
Import the one INCOMING signal which we recieve and process
"""
from .signals import (on_send_to_crocdoc,)


# -*- coding: utf-8 -*-
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType

from .models import CrocodocDocument
import signals as crocodoc_signals

import json
import logging
from bunch import Bunch
logger = logging.getLogger('django.request')


class CrocoDocConnectService(object):
    """
    Service to setup a crocodoc object
    """
    obj = None
    is_new = None

    def __init__(self, document_object, app_label, field_name='attachment', **kwargs):
        upload = kwargs.get('upload_immediately', False)
        reviewer = kwargs.get('reviewer', None)
        #
        # Get the content_type of the passed in model
        #
        content_type = ContentType.objects.get(model=document_object.__class__.__name__.lower(),
                                               app_label=app_label)
        #
        # Get or Create a new Crocodoc object associated with the document_object passed in
        #
        self.obj, self.is_new = CrocodocDocument.objects.get_or_create(content_object_type=content_type,
                                                                       object_id=document_object.pk,
                                                                       object_attachment_fieldname=field_name,
                                                                       reviewer=reviewer)
        if upload in [True, 'true', 1, '1']:
            # cause an upload to happen
            self.generate()

    def generate(self):
        self.obj.crocodoc_service.uuid

    def download_pdf_file(self):
        return self.obj.crocodoc_service.download_pdf_file()



class CrocodocWebhookService(object):
    payload = None

    def __init__(self, payload, *args, **kwargs):
        self.user = kwargs.get('user')
        self.payload = json.loads(payload)
        self.items = [Bunch(**i) for i in self.payload]

    def process(self):
        page = None
        processed = []

        for c, i in enumerate(self.items):

            event = i.get('event')
            event_type = i.get('type')

            if i.get('page') is not None:
                page = i.get('page')

            logger.info("{event} is of type {event_type} on page: {page}".format(event_type=event_type,
                                                                                 event=event,
                                                                                 page=page))
            if event == 'comment.create':
                i = CrocodocCommentCreateEvent(page=page, **i)

            if event == 'comment.update':
                i = CrocodocCommentReplyEvent(page=page, **i)

            elif event == 'comment.delete':
                i = CrocodocCommentDeleteEvent(**i)

            elif event in ['annotation.create', 'annotation.delete']:

                if event_type == 'textbox':
                    i = CrocodocAnnotationTextboxEvent(**i)

                elif event_type == 'highlight':
                    i = CrocodocAnnotationHighlightEvent(**i)

                elif event_type == 'strikeout':
                    i = CrocodocAnnotationStrikeoutEvent(**i)

                elif event_type == 'drawing':
                    i = CrocodocAnnotationDrawingEvent(**i)

                elif event_type == 'point':
                    i = CrocodocAnnotationPointEvent(**i)

            processed.append(i.process(sender=self) if hasattr(i, 'process') else None)

        return processed


class CrocodocBaseEvent(Bunch):
    signal = None
    _verb = None
    _deleted_verb = None
    _user = None
    _attachment = None
    label = 'Crocodoc Webhook Callback'
    content = None
    event = None
    type = None
    owner = None
    page = None
    doc = None
    uuid = None

    def __init__(self, *args, **kwargs):
        super(CrocodocBaseEvent, self).__init__(*args, **kwargs)
        self.__dict__.update(kwargs)  # @TODO ugly ugly ugly fix this

    @property
    def user(self):
        """ Crocodoc provides userid as string(pk,user_name)"""
        if self._user is None:
            pk, full_name = self.owner.split(',')
            pk = int(pk)
            self._user = User.objects.get(pk=pk)
        return self._user

    @property
    def attachment(self):
        if self._attachment is None:
            from .models import CrocodocDocument
            try:

                self._attachment = CrocodocDocument.objects.get(uuid=str(self.doc))  # must call str to make filter happen

            except CrocodocDocument.DoesNotExist:
                logger.critical('CrocodocDocument.DoesNotExist: %s' % self.doc)

        return self._attachment

    @property
    def verb(self):
        if self.event is not None and 'delete' in self.event:
            return self._deleted_verb
        else:
            return self._verb

    def process(self, sender):
        # try:
        document = self.attachment
        target = filename = None

        if document is not None:
            target = document.source_object
            #
            # We allow the fieldname to be variable and is specified at the model
            # level by setting object_attachment_fieldname but default to attachment
            #
            filename = getattr(target, document.object_attachment_fieldname, 'attachment').name

            user_pk, user_name = self.owner.split(',')  # crocodoc provides as pk,name

            self.signal.send(sender=sender,
                             verb=self.verb,
                             document=document,
                             target=target,
                             attachment_name=filename,
                             user_info=(int(user_pk), user_name),
                             crocodoc_event=self.event,
                             uuid=getattr(self, 'uuid', None),
                             content=self.content)

            logger.info('Send signal: {signal} {verb}'.format(signal=self.signal.__class__.__name__, verb=self.verb))
            return True

        logger.error('No document could be found by that id: {doc}'.format(doc=str(self.doc)))
        return False


class CrocodocCommentCreateEvent(CrocodocBaseEvent):
    signal = crocodoc_signals.crocodoc_comment_create
    _verb = 'Commented on a Document'


class CrocodocCommentReplyEvent(CrocodocBaseEvent):
    signal = crocodoc_signals.crocodoc_comment_update
    _verb = 'Replied to a comment on a Document'


class CrocodocCommentDeleteEvent(CrocodocBaseEvent):
    signal = crocodoc_signals.crocodoc_comment_delete
    _verb = 'Deleted a Commented on a Document'


class CrocodocAnnotationHighlightEvent(CrocodocBaseEvent):
    signal = crocodoc_signals.crocodoc_annotation_highlight
    _verb = 'Hilighted some text on a Document'
    _deleted_verb = 'Deleted a Hilighted of some text on a Document'


class CrocodocAnnotationStrikeoutEvent(CrocodocBaseEvent):
    signal = crocodoc_signals.crocodoc_annotation_strikeout
    _verb = 'Struck out some text on a Document'
    _deleted_verb = 'Deleted the Strikeout of some text on a Document'


class CrocodocAnnotationTextboxEvent(CrocodocBaseEvent):
    signal = crocodoc_signals.crocodoc_annotation_textbox
    _verb = 'Added a text element on a Document'
    _deleted_verb = 'Deleted a text element on a Document'


class CrocodocAnnotationDrawingEvent(CrocodocBaseEvent):
    signal = crocodoc_signals.crocodoc_annotation_drawing
    _verb = 'Added a drawing element on a Document'
    _deleted_verb = 'Deleted a drawing element on a Document'


class CrocodocAnnotationPointEvent(CrocodocBaseEvent):
    signal = crocodoc_signals.crocodoc_annotation_point
    _verb = 'Added a point element to a Document'
    _deleted_verb = 'Deleted a point element on a Document'



# -*- coding: utf-8 -*-
from django.contrib import admin

from .models import CrocodocDocument


class CrocodocDocumentAdmin(admin.ModelAdmin):
    list_display = ('uuid',)
    list_filter = ('content_object_type',)
    search_fields = ('uuid',)


admin.site.register(CrocodocDocument, CrocodocDocumentAdmin)



# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url
from django.views.decorators.csrf import csrf_exempt

from .views import CrocdocCallbackView


urlpatterns = patterns('',
    url(r'^webhook/$', csrf_exempt(CrocdocCallbackView.as_view()), name='crocodoc_webhook_callback'),
)



# -*- coding: utf-8 -*-
"""
Webhook signals
"""
from django.dispatch import Signal, receiver

#
# Incoming events
#
send_to_crocodoc = Signal(providing_args=['document_object', 'app_label', 'field_name'])

#
# Outgoing Events
#
crocodoc_comment_create = Signal(providing_args=['verb', 'document', 'target', 'attachment_name', 'user_info', 'crocodoc_event', 'uuid', 'content'])
crocodoc_comment_update = Signal(providing_args=['verb', 'document', 'target', 'attachment_name', 'user_info', 'crocodoc_event', 'uuid', 'content'])
crocodoc_comment_delete = Signal(providing_args=['verb', 'document', 'target', 'attachment_name', 'user_info', 'crocodoc_event', 'uuid', 'content'])
crocodoc_annotation_highlight = Signal(providing_args=['verb', 'document', 'target', 'attachment_name', 'user_info', 'crocodoc_event', 'uuid', 'content'])
crocodoc_annotation_strikeout = Signal(providing_args=['verb', 'document', 'target', 'attachment_name', 'user_info', 'crocodoc_event', 'uuid', 'content'])
crocodoc_annotation_textbox = Signal(providing_args=['verb', 'document', 'target', 'attachment_name', 'user_info', 'crocodoc_event', 'uuid', 'content'])
crocodoc_annotation_drawing = Signal(providing_args=['verb', 'document', 'target', 'attachment_name', 'user_info', 'crocodoc_event', 'uuid', 'content'])
crocodoc_annotation_point = Signal(providing_args=['verb', 'document', 'target', 'attachment_name', 'user_info', 'crocodoc_event', 'uuid', 'content'])


@receiver(send_to_crocodoc)
def on_send_to_crocdoc(sender, document_object, app_label, field_name='attachment', upload_immediately=True, **kwargs):
    """
    Signal to create a crocodoc object
    """
    from .services import CrocoDocConnectService
    kwargs.update({'upload_immediately': upload_immediately})

    service = CrocoDocConnectService(document_object=document_object,
                                     app_label=app_label,
                                     field_name=field_name,
                                     **kwargs)



# -*- coding: utf-8 -*-
import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "django-crocodoc",
    packages=['dj_crocodoc'],
    description = ("Django app for integrating with CrocoDoc"),
    version = "0.1.5",
    author = "Ross Crawford-d'Heureuse",
    author_email = "ross@lawpal.com",
    license = "MIT",
    keywords = "django crocdoc app",
    url = "https://github.com/rosscdh/django-crocodoc",
    install_requires = [
        'crocodoc',
        'django-braces',
        'django-jsonfield',
        'django-uuidfield',
        'bunch',
        'HTTPretty',  # for testing
    ]
)



# -*- coding: utf-8 -*-
from django.core.validators import URLValidator
from django.core.files.storage import default_storage
from django.core.exceptions import ValidationError

from . import CROCDOC_API_KEY
import logging
import crocodoc as CROCODOC_BASE_SERVICE
logger = logging.getLogger('django.request')


CROCODOC_BASE_SERVICE.api_token = CROCDOC_API_KEY


class CrocodocService(object):
    """
    Service to manage uploading and general attribs of corcdoc attachments
    """
    attachment = None
    session = None

    def __init__(self, attachment, source_object, attachment_field_name, *args, **kwargs):
        logger.info('Init CrocodocAttachmentService.__init__ for attachment: {pk}'.format(pk=attachment.pk))
        self.attachment = attachment
        self.source_object = source_object
        self.attachment_field_name = attachment_field_name

    @property
    def uuid(self):
        """
        Calling this property will initiate an upload of the doc,
        if it has not already been uploaded (i.e. we have a crocodoc uuid in the json data)
        """
        if self.attachment.uuid is None:

            crocodoc_uuid = self.upload_document()
            if crocodoc_uuid is None:
                raise Exception('Could not upload document to Crocodoc')

            else:
                logger.info('CrocodocAttachmentService.uuid: {uuid}'.format(uuid=crocodoc_uuid))
                self.attachment.uuid = crocodoc_uuid
                self.attachment.save(update_fields=['uuid'])

        return self.attachment.uuid

    def session_key(self, **kwargs):
        self.session = CROCODOC_BASE_SERVICE.session.create(str(self.uuid), **kwargs)
        logger.info('Session start:crocodoc: {session}'.format(session=self.session))
        return self.session

    def upload_document(self):
        validate = URLValidator()
        url = self.attachment.get_url()

        try:
            #
            # validate taht we are using a url
            #
            validate(url)
            logger.info('Upload url:file to crocodoc: {url}'.format(url=url))
            #
            # @TODO download the file locally? instead of trying to upload as a url?
            #
            return CROCODOC_BASE_SERVICE.document.upload(url=url)

        except ValidationError:
            #
            # was not a url is a patch
            #
            logger.info('Upload file to crocodoc: {url}'.format(url=url))
            return CROCODOC_BASE_SERVICE.document.upload(file=default_storage.open(url))
        else:
            return None

    def download_pdf_file(self):
        """
        https://github.com/crocodoc/crocodoc-python
        To download a document, use crocodoc.download.document(). Pass in the 
        uuid, an optional boolean of whether or not the file should be downloaded
        as a PDF, an optional boolean or whether or not the file should be annotated,
        and an optional filter string. This function returns the file contents as a string, which you probably want to save to a file.
        """
        return CROCODOC_BASE_SERVICE.download.document(uuid=self.attachment.uuid, pdf=True, annotated=False, user_filter='all')

    def view_url(self, **kwargs):
        """
        Please see: https://crocodoc.com/docs/api/#session-create
        for a list of the session kwargs that are accepted
        filter=[1,5,3,7] a list of user ids that can see the document commments
        """
        url = 'https://crocodoc.com/view/{session_key}'.format(session_key=self.session_key(**kwargs))
        logger.info('provide crocodoc view_url: {url}'.format(url=url))
        return url

    def remove(self):
        # delete from crocodoc based on uuid
        deleted = CROCODOC_BASE_SERVICE.document.delete(self.attachment.crocodoc_uuid)

        if deleted:
            logger.info('Deleted crocodoc file: {pk} - {uuid}'.format(pk=self.attachment.pk, uuid=self.attachment.crocodoc_uuid))

        else:
            logger.error('Could not Delete crocodoc file: {pk} - {uuid}'.format(pk=self.attachment.pk, uuid=self.attachment.crocodoc_uuid))

    def process(self):
        logger.info('Start CrocodocAttachmentService.process')
        return str(self.uuid)
